@@ -7,6 +7,8 @@
 #include "imagewriter.h"
 #include <iostream>
 
+extern void qt_imageTransform(QImage &src, QImageIOHandler::Transformations transforms);
+
 using namespace std;
 
 namespace ffmpegthumbnailer
@@ -16,7 +18,7 @@ ImageWriter::ImageWriter()
 {
 }
 
-void ImageWriter::writeFrame(VideoFrame& frame,  QImage& image)
+void ImageWriter::writeFrame(VideoFrame& frame, QImage& image, const QImageIOHandler::Transformations transformations)
 {
     QImage previewImage(frame.width, frame.height, QImage::Format_RGB888);
     for (quint32 y = 0; y < frame.height; y++) {
@@ -24,6 +26,8 @@ void ImageWriter::writeFrame(VideoFrame& frame,  QImage& image)
         memcpy(previewImage.scanLine(y), &frame.frameData[y*frame.lineSize], frame.width*3);
     }
 
+    qt_imageTransform(previewImage, transformations);
+
     image = previewImage;
 }
 }
